@@ -1,19 +1,20 @@
-#!/usr/bin/python
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """notebook.py - classes for the notebook widget"""
 
+from functools import cmp_to_key
 from gi.repository import GObject
 from gi.repository import Gtk
+from gi.repository import Gdk
 from gi.repository import Gio
 
-from terminator import Terminator
-from config import Config
-from factory import Factory
-from container import Container
-from editablelabel import EditableLabel
-from translation import _
-from util import err, dbg, enumerate_descendants, make_uuid
+from .terminator import Terminator
+from .config import Config
+from .factory import Factory
+from .container import Container
+from .editablelabel import EditableLabel
+from .translation import _
+from .util import err, dbg, enumerate_descendants, make_uuid
 
 class Notebook(Container, Gtk.Notebook):
     """Class implementing a Gtk.Notebook container"""
@@ -35,8 +36,12 @@ class Notebook(Container, Gtk.Notebook):
         GObject.type_register(Notebook)
         self.register_signals(Notebook)
         self.connect('switch-page', self.deferred_on_tab_switch)
+        self.connect('scroll-event', self.on_scroll_event)
+        self.connect('create-window', self.create_window_detach)
         self.configure()
 
+        self.set_can_focus(False)
+
         child = window.get_child()
         window.remove(child)
         window.add(self)
@@ -63,7 +68,7 @@ class Notebook(Container, Gtk.Notebook):
             pos = getattr(Gtk.PositionType, self.config['tab_position'].upper())
             self.set_tab_pos(pos)
 
-        for tab in xrange(0, self.get_n_pages()):
+        for tab in range(0, self.get_n_pages()):
             label = self.get_tab_label(self.get_nth_page(tab))
             label.update_angle()
 
@@ -73,11 +78,28 @@ class Notebook(Container, Gtk.Notebook):
 #        self.modify_style(style)
         self.last_active_term = {}
 
+    def create_window_detach(self, notebook, widget, x, y):
+        """Create a window to contain a detached tab"""
+        dbg('creating window for detached tab: %s' % widget)
+        maker = Factory()
+
+        window = maker.make('Window')
+        window.move(x, y)
+        size = self.window.get_size()
+        window.resize(size.width, size.height)
+
+        self.detach_tab(widget)
+        self.disconnect_child(widget)
+        self.hoover()
+        window.add(widget)
+
+        window.show_all()
+
     def create_layout(self, layout):
         """Apply layout configuration"""
         def child_compare(a, b):
-            order_a = children[a]['order']
-            order_b = children[b]['order']
+            order_a = int(children[a]['order'])
+            order_b = int(children[b]['order'])
 
             if (order_a == order_b):
                 return 0
@@ -86,7 +108,7 @@ class Notebook(Container, Gtk.Notebook):
             if (order_a > order_b):
                 return 1
 
-        if not layout.has_key('children'):
+        if 'children' not in layout:
             err('layout specifies no children: %s' % layout)
             return
 
@@ -97,8 +119,8 @@ class Notebook(Container, Gtk.Notebook):
             return
 
         num = 0
-        keys = children.keys()
-        keys.sort(child_compare)
+        keys = list(children.keys())
+        keys = sorted(keys, key=cmp_to_key(child_compare))
 
         for child_key in keys:
             child = children[child_key]
@@ -120,7 +142,7 @@ class Notebook(Container, Gtk.Notebook):
                 # This page does not yet exist, so make it
                 self.newtab(children[child_key])
                 page = self.get_nth_page(num)
-            if layout.has_key('labels'):
+            if 'labels' in layout:
                 labeltext = layout['labels'][num]
                 if labeltext and labeltext != "None":
                     label = self.get_tab_label(page)
@@ -131,7 +153,7 @@ class Notebook(Container, Gtk.Notebook):
                 self.last_active_term[page] = make_uuid(layout['last_active_term'][num])
             num = num + 1
 
-        if layout.has_key('active_page'):
+        if 'active_page' in layout:
             # Need to do it later, or layout changes result
             GObject.idle_add(self.set_current_page, int(layout['active_page']))
         else:
@@ -160,13 +182,16 @@ class Notebook(Container, Gtk.Notebook):
         if not sibling:
             sibling = maker.make('terminal')
             sibling.set_cwd(cwd)
+            if self.config['always_split_with_profile']:
+                sibling.force_set_profile(None, widget.get_profile())
             sibling.spawn_child()
             if widget.group and self.config['split_to_group']:
                 sibling.set_group(None, widget.group)
-        if self.config['always_split_with_profile']:
+        elif self.config['always_split_with_profile']:
             sibling.force_set_profile(None, widget.get_profile())
 
         self.insert_page(container, None, page_num)
+        self.set_tab_detachable(container, True)
         self.child_set_property(container, 'tab-expand', True)
         self.child_set_property(container, 'tab-fill', True)
         self.set_tab_reorderable(container, True)
@@ -198,7 +223,7 @@ class Notebook(Container, Gtk.Notebook):
         """Remove a widget from the container"""
         page_num = self.page_num(widget)
         if page_num == -1:
-            err('%s not found in Notebook. Actual parent is: %s' % 
+            err('%s not found in Notebook. Actual parent is: %s' %
                     (widget, widget.get_parent()))
             return(False)
         self.remove_page(page_num)
@@ -229,7 +254,7 @@ class Notebook(Container, Gtk.Notebook):
     def get_children(self):
         """Return an ordered list of our children"""
         children = []
-        for page in xrange(0,self.get_n_pages()):
+        for page in range(0,self.get_n_pages()):
             children.append(self.get_nth_page(page))
         return(children)
 
@@ -243,19 +268,23 @@ class Notebook(Container, Gtk.Notebook):
             widget = maker.make('Terminal')
             if cwd:
                 widget.set_cwd(cwd)
+            if profile and self.config['always_split_with_profile']:
+                widget.force_set_profile(None, profile)
             widget.spawn_child(debugserver=debugtab)
-        if profile and self.config['always_split_with_profile']:
+        elif profile and self.config['always_split_with_profile']:
             widget.force_set_profile(None, profile)
 
         signals = {'close-term': self.wrapcloseterm,
                    'split-horiz': self.split_horiz,
                    'split-vert': self.split_vert,
                    'title-change': self.propagate_title_change,
-                   'unzoom': self.unzoom,
                    'tab-change': top_window.tab_change,
                    'group-all': top_window.group_all,
                    'group-all-toggle': top_window.group_all_toggle,
                    'ungroup-all': top_window.ungroup_all,
+                   'group-win': top_window.group_win,
+                   'group-win-toggle': top_window.group_win_toggle,
+                   'ungroup-win': top_window.ungroup_win,
                    'group-tab': top_window.group_tab,
                    'group-tab-toggle': top_window.group_tab_toggle,
                    'ungroup-tab': top_window.ungroup_tab,
@@ -272,13 +301,13 @@ class Notebook(Container, Gtk.Notebook):
                     handler = handler[0]
                 self.connect_child(widget, signal, handler, *args)
 
-        if metadata and metadata.has_key('tabnum'):
+        if metadata and 'tabnum' in metadata:
             tabpos = metadata['tabnum']
         else:
             tabpos = -1
 
         label = TabLabel(self.window.get_title(), self)
-        if metadata and metadata.has_key('label'):
+        if metadata and 'label' in metadata:
             dbg('creating TabLabel with text: %s' % metadata['label'])
             label.set_custom_label(metadata['label'])
         label.connect('close-clicked', self.closetab)
@@ -288,6 +317,7 @@ class Notebook(Container, Gtk.Notebook):
 
         dbg('inserting page at position: %s' % tabpos)
         self.insert_page(widget, None, tabpos)
+        self.set_tab_detachable(widget, True)
 
         if maker.isinstance(widget, 'Terminal'):
             containers, objects = ([], [widget])
@@ -312,12 +342,12 @@ class Notebook(Container, Gtk.Notebook):
 
     def wrapcloseterm(self, widget):
         """A child terminal has closed"""
-        dbg('Notebook::wrapcloseterm: called on %s' % widget)
+        dbg('called on %s' % widget)
         if self.closeterm(widget):
-            dbg('Notebook::wrapcloseterm: closeterm succeeded')
+            dbg('closeterm succeeded')
             self.hoover()
         else:
-            dbg('Notebook::wrapcloseterm: closeterm failed')
+            dbg('closeterm failed')
 
     def closetab(self, widget, label):
         """Close a tab"""
@@ -328,7 +358,7 @@ class Notebook(Container, Gtk.Notebook):
             err('TabLabel::closetab: called on non-Notebook: %s' % widget)
             return
 
-        for i in xrange(0, nb.get_n_pages() + 1):
+        for i in range(0, nb.get_n_pages() + 1):
             if label == nb.get_tab_label(nb.get_nth_page(i)):
                 tabnum = i
                 break
@@ -341,7 +371,7 @@ class Notebook(Container, Gtk.Notebook):
         child = nb.get_nth_page(tabnum)
 
         if maker.isinstance(child, 'Terminal'):
-            dbg('Notebook::closetab: child is a single Terminal')
+            dbg('child is a single Terminal')
             del nb.last_active_term[child]
             child.close()
             # FIXME: We only do this del and return here to avoid removing the
@@ -349,7 +379,7 @@ class Notebook(Container, Gtk.Notebook):
             del(label)
             return
         elif maker.isinstance(child, 'Container'):
-            dbg('Notebook::closetab: child is a Container')
+            dbg('child is a Container')
             result = self.construct_confirm_close(self.window, _('tab'))
 
             if result == Gtk.ResponseType.ACCEPT:
@@ -364,7 +394,7 @@ class Notebook(Container, Gtk.Notebook):
                         Gtk.main_iteration()
                 return
             else:
-                dbg('Notebook::closetab: user cancelled request')
+                dbg('user cancelled request')
                 return
         else:
             err('Notebook::closetab: child is unknown type %s' % child)
@@ -404,7 +434,7 @@ class Notebook(Container, Gtk.Notebook):
         if not label:
             err('Notebook::update_tab_label_text: %s not found' % widget)
             return
-        
+
         label.set_label(text)
 
     def hoover(self):
@@ -428,7 +458,7 @@ class Notebook(Container, Gtk.Notebook):
             del(self)
             # Find the last terminal in the new parent and give it focus
             terms = parent.get_visible_terminals()
-            terms.keys()[-1].grab_focus()
+            list(terms.keys())[-1].grab_focus()
 
     def page_num_descendant(self, widget):
         """Find the tabnum of the tab containing a widget at any level"""
@@ -458,7 +488,7 @@ class Notebook(Container, Gtk.Notebook):
         if self.terminator.doing_layout == True:
             return
         last_active_term = {}
-        for tabnum in xrange(0, self.get_n_pages()):
+        for tabnum in range(0, self.get_n_pages()):
             nth_page = self.get_nth_page(tabnum)
             if nth_page in self.last_active_term:
                 last_active_term[nth_page] = self.last_active_term[nth_page]
@@ -468,7 +498,7 @@ class Notebook(Container, Gtk.Notebook):
         """Prime a single idle tab switch signal, using the most recent set of params"""
         tabs_last_active_term = self.last_active_term.get(self.get_nth_page(page_num),  None)
         data = {'tabs_last_active_term':tabs_last_active_term}
-        
+
         self.pending_on_tab_switch_args = (notebook, page,  page_num,  data)
         if self.pending_on_tab_switch == True:
             return
@@ -486,7 +516,58 @@ class Notebook(Container, Gtk.Notebook):
         tabs_last_active_term = data['tabs_last_active_term']
         if tabs_last_active_term:
             term = self.terminator.find_terminal_by_uuid(tabs_last_active_term.urn)
-            GObject.idle_add(term.ensure_visible_and_focussed)
+            # if we can't find a last active term we must be starting up
+            if term is not None:
+                GObject.idle_add(term.ensure_visible_and_focussed)
+        return True
+
+    def on_scroll_event(self, notebook, event):
+        '''Handle scroll events for scrolling through tabs'''
+        #print "self: %s" % self
+        #print "event: %s" % event
+        child = self.get_nth_page(self.get_current_page())
+        if child == None:
+            print("Child = None,  return false")
+            return False
+
+        event_widget = Gtk.get_event_widget(event)
+
+        if event_widget == None or \
+           event_widget == child or \
+           event_widget.is_ancestor(child):
+            print("event_widget is wrong one,  return false")
+            return False
+
+        # Not sure if we need these. I don't think wehave any action widgets
+        # at this point.
+        action_widget = self.get_action_widget(Gtk.PackType.START)
+        if event_widget == action_widget or \
+           (action_widget != None and event_widget.is_ancestor(action_widget)):
+            return False
+        action_widget = self.get_action_widget(Gtk.PackType.END)
+        if event_widget == action_widget or \
+           (action_widget != None and event_widget.is_ancestor(action_widget)):
+            return False
+
+        if event.direction in [Gdk.ScrollDirection.RIGHT,
+                               Gdk.ScrollDirection.DOWN]:
+            self.next_page()
+        elif event.direction in [Gdk.ScrollDirection.LEFT,
+                                 Gdk.ScrollDirection.UP]:
+            self.prev_page()
+        elif event.direction == Gdk.ScrollDirection.SMOOTH:
+            if self.get_tab_pos() in [Gtk.PositionType.LEFT,
+                                      Gtk.PositionType.RIGHT]:
+                if event.delta_y > 0:
+                    self.next_page()
+                elif event.delta_y < 0:
+                    self.prev_page()
+            elif self.get_tab_pos() in [Gtk.PositionType.TOP,
+                                        Gtk.PositionType.BOTTOM]:
+                if event.delta_x > 0:
+                    self.next_page()
+                elif event.delta_x < 0:
+                    self.prev_page()
         return True
 
 class TabLabel(Gtk.HBox):
@@ -511,6 +592,8 @@ class TabLabel(Gtk.HBox):
         self.terminator = Terminator()
         self.config = Config()
 
+        self.connect("button-press-event", self.on_button_pressed)
+
         self.label = EditableLabel(title)
         self.update_angle()
 
@@ -526,9 +609,9 @@ class TabLabel(Gtk.HBox):
     def get_label(self):
         return self.label.get_text()
 
-    def set_custom_label(self, text):
+    def set_custom_label(self, text, force=False):
         """Set a permanent label as if the user had edited it"""
-        self.label.set_text(text)
+        self.label.set_text(text, force=force)
         self.label.set_custom()
 
     def get_custom_label(self):
@@ -558,7 +641,7 @@ class TabLabel(Gtk.HBox):
         if not self.icon:
             self.icon = Gio.ThemedIcon.new_with_default_fallbacks("window-close-symbolic")
             self.icon = Gtk.Image.new_from_gicon(self.icon, Gtk.IconSize.MENU)
-            
+
         self.button.set_focus_on_click(False)
         self.button.set_relief(Gtk.ReliefStyle.NONE)
 #        style = Gtk.RcStyle()  # FIXME FOR GTK3 how to do it there? actually do we really want to override the theme?
@@ -593,4 +676,8 @@ class TabLabel(Gtk.HBox):
         """The close button has been clicked. Destroy the tab"""
         self.emit('close-clicked', self)
 
+    def on_button_pressed(self, _widget, event):
+        if event.button == 2:
+            self.on_close(_widget)
+
 # vim: set expandtab ts=4 sw=4:
