@@ -7,7 +7,6 @@ from gi.repository import Gtk
 
 from .version import APP_NAME
 from .translation import _
-from .encoding import TerminatorEncoding
 from .terminator import Terminator
 from .util import err, dbg, spawn_new_terminator
 from .config import Config
@@ -110,6 +109,10 @@ class TerminalPopupMenu(object):
 
         menu.append(Gtk.SeparatorMenuItem())
 
+        item = Gtk.ImageMenuItem.new_with_mnemonic(_('Set W_indow Title'))
+        item.connect('activate', lambda x: terminal.key_edit_window_title())
+        menu.append(item)
+        
         if not terminal.is_zoomed():
             item = Gtk.ImageMenuItem.new_with_mnemonic(_('Split H_orizontally'))
             image = Gtk.Image()
@@ -217,7 +220,6 @@ class TerminalPopupMenu(object):
                 item.connect('activate', terminal.force_set_profile, profile)
                 submenu.append(item)
 
-        self.add_encoding_items(menu)
         self.add_layout_launcher(menu)
 
         try:
@@ -252,69 +254,3 @@ class TerminalPopupMenu(object):
                 item = Gtk.MenuItem(layout)
                 item.connect('activate', lambda x: spawn_new_terminator(self.terminator.origcwd, ['-u', '-l', layout]))
                 submenu.append(item)
-
-    def add_encoding_items(self, menu):
-        """Add the encoding list to the menu"""
-        terminal = self.terminal
-        active_encodings = terminal.config['active_encodings']
-        item = Gtk.MenuItem.new_with_mnemonic(_("Encodings"))
-        menu.append (item)
-        submenu = Gtk.Menu ()
-        item.set_submenu (submenu)
-        encodings = TerminatorEncoding ().get_list ()
-        encodings.sort (key=lambda x: x[2].lower ())
-
-        current_encoding = terminal.vte.get_encoding ()
-        group = None
-
-        if current_encoding not in active_encodings:
-            active_encodings.insert (0, _(current_encoding))
-
-        for encoding in active_encodings:
-            if encoding == terminal.default_encoding:
-                extratext = " (%s)" % _("Default")
-            elif encoding == current_encoding and \
-                 terminal.custom_encoding == True:
-                extratext = " (%s)" % _("User defined")
-            else:
-                extratext = ""
-    
-            radioitem = Gtk.RadioMenuItem (_(encoding) + extratext, group)
-    
-            if encoding == current_encoding:
-                radioitem.set_active (True)
-    
-            if group is None:
-                group = radioitem
-    
-            radioitem.connect ('activate', terminal.on_encoding_change, 
-                               encoding)
-            submenu.append (radioitem)
-    
-        item = Gtk.MenuItem.new_with_mnemonic(_("Other Encodings"))
-        submenu.append (item)
-        #second level
-    
-        submenu = Gtk.Menu ()
-        item.set_submenu (submenu)
-        group = None
-    
-        for encoding in encodings:
-            if encoding[1] in active_encodings:
-                continue
-
-            if encoding[1] is None:
-                label = "%s %s" % (encoding[2], terminal.vte.get_encoding ())
-            else:
-                label = "%s %s" % (encoding[2], encoding[1])
-    
-            radioitem = Gtk.RadioMenuItem (label, group)
-            if group is None:
-                group = radioitem
-    
-            if encoding[1] == current_encoding:
-                radioitem.set_active (True)
-
-            radioitem.connect ('activate', terminal.on_encoding_change, 
-                               encoding[1])
-            submenu.append (radioitem)
