@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """titlebar.py - classes necessary to provide a terminal title bar"""
@@ -9,11 +8,11 @@ from gi.repository import Pango
 import random
 import itertools
 
-from version import APP_NAME
-from util import dbg
-from terminator import Terminator
-from editablelabel import EditableLabel
-from translation import _
+from .version import APP_NAME
+from .util import dbg
+from .terminator import Terminator
+from .editablelabel import EditableLabel
+from .translation import _
 
 # pylint: disable-msg=R0904
 # pylint: disable-msg=W0613
@@ -105,10 +104,15 @@ class Titlebar(Gtk.EventBox):
     def update(self, other=None):
         """Update our contents"""
         default_bg = False
-        if self.config['title_hide_sizetext']:
-            self.label.set_text("%s" % self.termtext)
-        else:
-            self.label.set_text("%s %s" % (self.termtext, self.sizetext))
+
+        temp_heldtext_str = ''
+        temp_sizetext_str = ''
+
+        if self.terminal.is_held_open:
+            temp_heldtext_str = _('[INACTIVE: Right-Click for Relaunch option] ')
+        if not self.config['title_hide_sizetext']:
+            temp_sizetext_str = " %s" % (self.sizetext)
+        self.label.set_text("%s%s%s" % (temp_heldtext_str, self.termtext, temp_sizetext_str))
 
         if (not self.config['title_use_system_font']) and self.config['title_font']:
             title_font = Pango.FontDescription(self.config['title_font'])
@@ -279,7 +283,7 @@ class Titlebar(Gtk.EventBox):
     def groupentry_activate(self, widget):
         """Actually cause a group to be created"""
         groupname = self.groupentry.get_text() or None
-        dbg('Titlebar::groupentry_activate: creating group: %s' % groupname)
+        dbg('creating group: %s' % groupname)
         self.groupentry_cancel(None, None)
         last_focused_term=self.terminator.last_focused_term
         if self.terminal.targets_for_new_group:
